import { Database, } from "@orbitdb/core";
const type = "feed";
const Feed = () => async ({ ipfs, identity, address, name, access, directory, meta, headsStorage, entryStorage, indexStorage, referencesCount, syncAutomatically, onUpdate, }) => {
    const database = await Database({
        ipfs,
        identity,
        address,
        name,
        access,
        directory,
        meta,
        headsStorage,
        entryStorage,
        indexStorage,
        referencesCount,
        syncAutomatically,
        onUpdate,
    });
    const { addOperation, log } = database;
    const add = async (value) => {
        return addOperation({ op: "ADD", key: null, value });
    };
    const remove = async (hash) => {
        return addOperation({ op: "DEL", key: null, value: hash });
    };
    const iterator = async function* ({ amount, } = {}) {
        const vals = {};
        let count = 0;
        for await (const entry of log.traverse()) {
            const { op, value } = entry.payload;
            const { hash } = entry;
            if (op === "ADD" && !vals[hash]) {
                count++;
                const hash = entry.hash;
                vals[hash] = true;
                yield { value, hash };
            }
            else if (op === "DEL" && !vals[value]) {
                vals[value] = true;
            }
            if (amount !== undefined && count >= amount) {
                break;
            }
        }
    };
    const all = async () => {
        const values = [];
        for await (const entry of iterator()) {
            values.unshift(entry);
        }
        return values;
    };
    return {
        ...database,
        type,
        add,
        remove,
        iterator,
        all,
    };
};
Feed.type = type;
export default Feed;
//# sourceMappingURL=feed.js.map